package cn.zyjblogs.design.singleton;

/**
 * 懒汉式
 * 虽然也能达到按需初始化的目的，但却带来了线程不安全问题
 */
public class Mgr05 {
    private static Mgr05 INSTANCE;

    private Mgr05() {
    }

    /**
     * 线程不安全,但是要上锁，效率降低了
     *
     * @return
     */
    public static  Mgr05 getInstance() {
        if (INSTANCE == null) {
            //通过减小同步代码块的方式提高效率，然后可不行
            synchronized (Mgr05.class) {
                try {
                    Thread.sleep(1);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
            INSTANCE = new Mgr05();
        }
        return INSTANCE;
    }

    public void m() {
        System.out.println("m");
    }

    public static void main(String[] args) {
        for (int i = 0; i < 100; i++) {
            new Thread(() -> {
                System.out.println(Mgr05.getInstance().hashCode());
            }).start();
        }
    }
}
