package cn.zyjblogs.design.singleton;

/**
 * 懒汉式
 * 虽然也能达到按需初始化的目的，但却带来了线程不安全问题
 */
public class Mgr06 {
    //volatile 防止指令从拍 JIT
    private static volatile Mgr06 INSTANCE;

    private Mgr06() {
    }

    /**
     * 线程安全,但是要上锁
     *
     * @return
     */
    public static Mgr06 getInstance() {
        if (INSTANCE == null) {
            //通过减小同步代码块的方式提高效率，
            synchronized (Mgr06.class) {
                //双重检查
                if (INSTANCE != null) {
                    return INSTANCE;
                }
                try {
                    Thread.sleep(1);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                INSTANCE = new Mgr06();
            }
        }
        return INSTANCE;
    }

    public void m() {
        System.out.println("m");
    }

    public static void main(String[] args) {
        for (int i = 0; i < 100; i++) {
            new Thread(() -> {
                System.out.println(Mgr06.getInstance().hashCode());
            }).start();
        }
    }
}
